==> kite-1.0b7.tar.gz: kite-1.0b7/modules/System/directory.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>
#include "kite_object.h"
#include "vm/kite_vm.h"

#define CHECK_FILE_OPEN \
    if (this->builtin_data.filevalue == NULL) { \
        kite_object_t *exc = \
            kite_new_exception(thd, "System.exceptions.FileError", \
                               "Directory is not open"); \
        kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE); \
        return; \
    }

/*****************************************************************************
 * Class constructor
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_construct) {
    kite_object_t *arg1 = args->builtin_data.listvalue.car;
    kite_object_t *file = kite_string_object(thd, arg1);

    this->builtin_data.filevalue = 
        opendir(file->builtin_data.stringvalue.string); 
    if (this->builtin_data.filevalue == NULL) {
        char *buf;
        buf = strerror(errno);
        kite_object_t *exc = 
            kite_new_exception(thd, "System.exceptions.FileError", buf);
        kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE);
        return;
    }

    kite_dereference_object(file);
}

/*****************************************************************************
 * Seek to given position in the directory
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_seek) {
    kite_object_t *arg1 = args->builtin_data.listvalue.car;
    kite_object_t *bytes = kite_int_object(thd, arg1);

    CHECK_FILE_OPEN;
    seekdir(this->builtin_data.filevalue, bytes->builtin_data.intvalue );
    kite_dereference_object(bytes);
}

/*****************************************************************************
 * Read directory entry.
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_read) {
    CHECK_FILE_OPEN;

    struct dirent *ent = readdir(this->builtin_data.filevalue);
    if (ent == 0) {
        kite_vm_return(thd, kite_new_null(thd));
    }
    kite_object_t *rval = 
        kite_new_string_with_length(
            thd, 
            ent->d_name, 
            strlen(ent->d_name)
        );
    kite_vm_return(thd, rval);
}

/*****************************************************************************
 * Close given directory
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_close) {
    KITE_NO_ARGS;

    CHECK_FILE_OPEN;
    closedir(this->builtin_data.filevalue);
    this->builtin_data.filevalue = NULL;
}

/*****************************************************************************
 * Get current position in directory
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_tell) {
    KITE_NO_ARGS;

    CHECK_FILE_OPEN;
    kite_vm_return(thd, kite_new_integer(thd, telldir(this->builtin_data.filevalue)));
}

KITE_MODULE_INITIALIZER(directory) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits =
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = kite_new_class(thread, inherits, "directory");
    kite_set_docstring(newclass, "Disk directory manipulation.");
    
    /* Add methods common to all classes. */
    kite_add_method(thread, newclass, "__construct__",
        kite_new_method_with_docs(thread, FUNC_COMPILED, Directory_construct, 
                                  "Opens a directory.", 1,
                                  "name", "Name of file to open."));
    kite_add_method(thread, newclass, "read", 
        kite_new_method_with_docs(thread, FUNC_COMPILED, Directory_read, 
                                  "Read a directory entry.", 0));
    kite_add_method(thread, newclass, "seek", 
        kite_new_method_with_docs(thread, FUNC_COMPILED, Directory_seek, 
                                  "Move current position in directory.", 1,
                                  "pos", "Position in directory to move to."));
    kite_add_method(thread, newclass, "tell", 
        kite_new_method_with_docs(thread, FUNC_COMPILED, Directory_tell, 
                                  "Get current position in directory.", 0));
    kite_add_method(thread, newclass, "close", 
        kite_new_method_with_docs(thread, FUNC_COMPILED, Directory_close, 
                                  "Closes directory and prevents further operations on this object.", 0));

    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "directory", 1, NULL);
    kite_set_property(thread, parent, "directory", newclass);
}

==> kite-1.0.0.tar.gz: kite-1.0.0/modules/System/directory.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#ifndef WIN32
#include <dirent.h>
#endif /* WIN32 */
#include "kite_object.h"
#include "vm/kite_vm.h"

#define CHECK_FILE_OPEN \
    if (this->builtin_data.filevalue == NULL) { \
        kite_object_t *exc = \
            kite_new_exception(thd, "System.exceptions.FileError", \
                               "Directory is not open"); \
        kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE); \
        return; \
    }

/*****************************************************************************
 * Class constructor
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_construct) {
#ifndef WIN32
    kite_object_t *arg1 = args->builtin_data.listvalue.car;
    kite_object_t *file = kite_string_object(thd, arg1);
    kite_object_t *exc;

    this->builtin_data.filevalue = 
        (FILE*)opendir(file->builtin_data.stringvalue.string); 
    if (this->builtin_data.filevalue == NULL) {
        char *buf;
        buf = strerror(errno);
        exc = 
            kite_new_exception(thd, "System.exceptions.FileError", buf);
        kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE);
        return;
    }

    kite_dereference_object(file);
#else
    kite_object_t *exc = kite_new_exception(thd, 
        "System.exceptions.NotImplemented", 
        "This operation is not implemented on Windows.");
    kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE);
#endif /* WIN32 */
}

/*****************************************************************************
 * Seek to given position in the directory
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_seek) {
#ifndef WIN32
    kite_object_t *arg1 = args->builtin_data.listvalue.car;
    kite_object_t *bytes = kite_int_object(thd, arg1);

    CHECK_FILE_OPEN;
    seekdir((DIR*)this->builtin_data.filevalue, bytes->builtin_data.intvalue);
    kite_dereference_object(bytes);
#else
    kite_object_t *exc = kite_new_exception(thd, 
        "System.exceptions.NotImplemented", 
        "This operation is not implemented on Windows.");
    kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE);
#endif /* WIN32 */
}

/*****************************************************************************
 * Read directory entry.
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_read) {
#ifndef WIN32
    struct dirent *ent;
    kite_object_t *rval;

    (void)args;
    CHECK_FILE_OPEN;
    ent = readdir((DIR*)this->builtin_data.filevalue);

    if (ent == 0) {
        kite_vm_return(thd, kite_new_null(thd));
    }
    rval = 
        kite_new_string_with_length(
            thd, 
            ent->d_name, 
            strlen(ent->d_name)
        );
    kite_vm_return(thd, rval);
#else
    kite_object_t *exc = kite_new_exception(thd, 
        "System.exceptions.NotImplemented", 
        "This operation is not implemented on Windows.");
    kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE);
#endif /* WIN32 */
}

/*****************************************************************************
 * Close given directory
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_close) {
#ifndef WIN32
    KITE_NO_ARGS;

    CHECK_FILE_OPEN;
    closedir((DIR*)this->builtin_data.filevalue);
    this->builtin_data.filevalue = NULL;
#else
    kite_object_t *exc = kite_new_exception(thd, 
        "System.exceptions.NotImplemented", 
        "This operation is not implemented on Windows.");
    kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE);
#endif /* WIN32 */
}

/*****************************************************************************
 * Get current position in directory
 ****************************************************************************/
KITE_CLASS_METHOD(Directory_tell) {
#ifndef WIN32
    KITE_NO_ARGS;

    CHECK_FILE_OPEN;
    kite_vm_return(thd, kite_new_integer(thd, telldir((DIR*)this->builtin_data.filevalue)));
#else
    kite_object_t *exc = kite_new_exception(thd, 
        "System.exceptions.NotImplemented", 
        "This operation is not implemented on Windows.");
    kite_vm_call_method(thd, exc, "throw", kite_new_list(thd), TRUE);
#endif /* WIN32 */
}

KITE_MODULE_INITIALIZER(directory) {
    /*
     * Create new class object. inherits is typically NULL for packages
     * and System.object.
     */
    kite_object_t *inherits =
        kite_dereference_and_load(thread, "System.object");
    kite_object_t *newclass = kite_new_class(thread, inherits, "directory");
    kite_set_docstring(newclass, "Disk directory manipulation.");
    
    /* Add methods common to all classes. */
    kite_add_method(thread, newclass, "__construct__",
        kite_new_method_compiled_with_docs(thread, Directory_construct, 
                                  "Opens a directory.", 1,
                                  "name", "Name of file to open."));
    kite_add_method(thread, newclass, "read", 
        kite_new_method_compiled_with_docs(thread, Directory_read, 
                                  "Read a directory entry.", 0));
    kite_add_method(thread, newclass, "seek", 
        kite_new_method_compiled_with_docs(thread, Directory_seek, 
                                  "Move current position in directory.", 1,
                                  "pos", "Position in directory to move to."));
    kite_add_method(thread, newclass, "tell", 
        kite_new_method_compiled_with_docs(thread, Directory_tell, 
                                  "Get current position in directory.", 0));
    kite_add_method(thread, newclass, "close", 
        kite_new_method_compiled_with_docs(thread, Directory_close, 
                                  "Closes directory and prevents further operations on this object.", 0));

    /* Add to parent to complete loading. */
    kite_add_property(thread, parent, "directory", 1, NULL);
    kite_set_property(thread, parent, "directory", newclass);
}
